package contrasts;

/**
 * 
 * This class is the object of a Confidence Interval. A Confidence Interval has
 * a center position (mean/contrast), a upper and a lower bound which are
 * generated by a delta.
 * 
 * A ConfidenceInterval is generated at a specific confidence level alpha.
 * 
 * Is is of interest to know if an interval include zero or overlap to another
 * interval. In fact these methods are implemented.
 * 
 * @author <a href="mailto:max.schubach@informatik.uni-tuebingen.de">Max
 *         Schubach</a>
 * 
 * @version 1.1
 * 
 */
public class ConfidenceInterval {

	/**
	 * Default constructor. generates default method
	 * 
	 * @param mean
	 * @param delta
	 * @param standardDeviation
	 */
	public ConfidenceInterval(double mean, double delta,
			double standardDeviation) {
		this.mean = mean;
		this.delta = delta;
		this.lowerBound = mean - delta;
		this.upperBound = mean + delta;
		this.standardDeviation = standardDeviation;
		this.method = new Method();
	}

	/**
	 * 
	 * Default constructor. Sets the given methods (needed for p-value)
	 * 
	 * @param mean
	 * @param delta
	 * @param standardDeviation
	 * @param method
	 */
	public ConfidenceInterval(double mean, double delta,
			double standardDeviation, Method method) {
		this.mean = mean;
		this.delta = delta;
		this.lowerBound = mean - delta;
		this.upperBound = mean + delta;
		this.standardDeviation = standardDeviation;
		this.method = method;
	}

	/**
	 * Center or mean of a confidence interval
	 */
	private double mean;
	/**
	 * The delta added (subtracted) from the delta. For contrasts teh upper and
	 * lower part of the interval has always the same size. In fact only one
	 * delta exists.
	 */
	private double delta;
	/**
	 * Highest value of the interval.
	 */
	private double upperBound;
	/**
	 * Lowest value of the interval.
	 */
	private double lowerBound;

	/**
	 * Standard deviation of the confidence interval. It is needed to generate a
	 * P-Value.
	 */
	private double standardDeviation;

	/**
	 * The Method class which provides the alpha and other constants. Basis or
	 * the generated interval.
	 */
	private Method method;

	/**
	 * @return The mean or center of the interval (contrast)
	 */
	public double getMean() {
		return this.mean;
	}

	// /**
	// *
	// *
	// * @param center
	// * the contrast to set
	// */
	// public void setContrast(double center) {
	// this.center = center;
	// }

	/**
	 * Getter method of delta.
	 * 
	 * @return the delta
	 */
	public double getDelta() {
		return this.delta;
	}

	// /**
	// * @param delta
	// * the delta to set
	// */
	// public void setDelta(double delta) {
	// this.delta = delta;
	// }

	/**
	 * Getter method of upperBound.
	 * 
	 * @return the upperBound
	 */
	public double getUpperBound() {
		return this.upperBound;
	}

	// /**
	// * @param upperBound
	// * the upperBound to set
	// */
	// public void setUpperBound(double upperBound) {
	// this.upperBound = upperBound;
	// }

	/**
	 * Getter method of lowerBound
	 * 
	 * @return the lowerBound
	 */
	public double getLowerBound() {
		return this.lowerBound;
	}

	/**
	 * Getter method of standardDeviation
	 * 
	 * @return the lowerBound
	 */
	public double getStandardDeviation() {
		return this.standardDeviation;
	}

	// /**
	// * @param lowerBound
	// * the lowerBound to set
	// */
	// public void setLowerBound(double lowerBound) {
	// this.lowerBound = lowerBound;
	// }

	/**
	 * Test if the ConfidenceInterval include zero.
	 * 
	 * @return <tt>True</tt> if the confidence interval include Zero. Else
	 *         <tt>false</tt>.
	 */
	public boolean includeZero() {
		return includeValue(0.0);
	}

	/**
	 * Tests if the value is included in the interval.
	 * 
	 * @param value
	 *            to test
	 * @return <tt>True</tt> if the value is included in the interval. Else
	 *         <tt>false</tt>.
	 */
	public boolean includeValue(double value) {
		if (this.upperBound >= value && this.lowerBound <= value) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Compare this interval with another interval if they Overlap.
	 * 
	 * @param ci
	 *            another confidence Interval
	 * @return <tt>True</tt> if the two intervals overlap, else <tt>false</tt>.
	 */
	public boolean overlapsInterval(ConfidenceInterval ci) {
		if (includeValue(ci.getUpperBound())
				|| includeValue(ci.getLowerBound())
				|| ci.includeValue(this.mean)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * This method computes a P-Value of a given value. Therefore the confidence
	 * interval is used (border is given alpha). It uses the given method in the
	 * Method class. (CI Should be constructed in the same way).
	 * 
	 * @param value
	 * @return The P-Value based on this interval.
	 */
	public double pValue(double value) {
		if (this.delta == 0.0) {
			return 1.0;
		}
		return this.method.pValue(this.mean, this.standardDeviation, value);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String out = "\"" + this.lowerBound + "\",\"" + this.mean + "\",\""
				+ this.upperBound + "\"";
		return out;
	}

}
